/*
 * Created on 28.10.2005
 */
package net.hoopajoo.cheeselab3;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import javax.swing.ComboBoxModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

/**
 * 
 * This class will implement Data models required for displaying an up to date
 * player list in various GUI components.
 * 
 * @author Antti Andreimann <Antti.Andreimann@mail.ee>
 * 
 */
public class CheesePlayersDataModel implements ComboBoxModel, CheeseMapListener {
    private static CheesePlayerData[] players = null;
    private static ArrayList listeners = new ArrayList();
    private static Object firstInstance = null;

    private Object selected = "Select a player";

    CheesePlayersDataModel() {
        /* Add listener to CheeseMap, if we are the first instance in existence */
        if( firstInstance == null ) {
            firstInstance = this;
            CheeseMap.addCheeseMapListener( this );
            rebuildList();
        }
    }

    private void rebuildList() {
        players = CheeseMap.map.get_players();
        Arrays.sort(players, new Comparator() {
                        public int compare(Object arg0, Object arg1) {
                            CheesePlayerData p1 = (CheesePlayerData)arg0;
                            CheesePlayerData p2 = (CheesePlayerData)arg1;

                            if( p1.online != p2.online )
                                return  p1.online ? -1 : 1;
                            else
                                return p1.name.compareTo( p2.name );
                        }
                    });

        if( selected instanceof CheesePlayerData ) {
            selected = CheeseMap.map.get_player( ( (CheesePlayerData)selected ).name );
        }

        notifyListeners();
    }

    public void setSelectedItem(Object anItem) {
        if( anItem == null)
            return;

        selected = anItem;

        notifyListeners();
    }

    public Object getSelectedItem() {
        return selected;
    }

    public int getSize() {
        return players.length;
    }

    public Object getElementAt(int index) {
        return players[index];
    }

    public void removeListDataListener(javax.swing.event.ListDataListener l) {
        listeners.remove( l );
    }

    public void addListDataListener(javax.swing.event.ListDataListener l) {
        listeners.add( l );
    }

    private void notifyListeners() {
        // no attempt at optimziation
        ListDataEvent le = new ListDataEvent( this,
                                              ListDataEvent.CONTENTS_CHANGED, 0, getSize() );
        for (int i = 0; i < listeners.size(); i++) {
            ((ListDataListener)listeners.get( i )).contentsChanged( le );
        }
    }

    public void playerChanged(CheesePlayerData player, CheesePlayerData oldplayer) {
        if( oldplayer == null || oldplayer.online != player.online ) {
            rebuildList();
        }
        else { /* Find the object in list and just update it */
            for( int i = 0; i < players.length; i++ ) {
                if( players[i].name.equals( player.name ) ) {
                    players[i] = player;
                    break;
                }
            }
        }
    }

    public void roomChanged(CheeseRoomData newroom, CheeseRoomData oldroom) { }
    public void roomDeleted(CheeseRoomData room) { }
    public void tileChanged(String fname) { }
    public void refreshStarted() { }
    public void refreshFinished() { }
    public void onlineChanged(boolean online) { }
}
